@@ -14,8 +14,8 @@
 extern "C" {
 #endif
 
-	FILE *
-		pexecvp(const char *file, char *const argv[], pid_t *pid, char *type);
+	FILE * pexecvp(const char *file, char *const argv[],
+                   pid_t *pid, const char *type);
 
 #ifdef __cplusplus
 }
